/*
 * IWT313 Control Program
 * Copyright (C) 2014 Tokyo Devices, I.W. Technology Firm, Inc.
 * http://tokyodevices.jp/
 * License: GNU GPL v2 (see License.txt)
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "hiddata.h"
#include "usbconfig.h"  /* for device VID, PID, vendor name and product name */

static char *usbErrorMessage(int errCode)
{
	static char buffer[80];

	switch(errCode)
	{
		case USBOPEN_ERR_ACCESS:      return "Access to device denied";
		case USBOPEN_ERR_NOTFOUND:    return "The specified device was not found";
		case USBOPEN_ERR_IO:          return "Communication error with device";
		default:
			sprintf(buffer, "Unknown USB error %d", errCode);
			return buffer;
	}

	return NULL;    /* not reached */
}

static usbDevice_t  *openDevice(char *serialNumber)
{
  usbDevice_t     *dev = NULL;
  
  // x_ID
  unsigned char   rawVid[2] = {USB_CFG_VENDOR_ID};
  
  // foCXID
  unsigned char   rawPid[2] = {USB_CFG_DEVICE_ID};
  
  // x_
  char            vendorName[] = {USB_CFG_VENDOR_NAME, 0};
  // v_Ng
  char			productName[] = {USB_CFG_DEVICE_NAME, 0};
  
  int             vid = rawVid[0] + 256 * rawVid[1];
  int             pid = rawPid[0] + 256 * rawPid[1];	
  int             err;
  
  if((err = usbhidOpenDevice(&dev, vid, vendorName, pid, productName, serialNumber, 0)) != 0){
    if( serialNumber != NULL ) {
      fprintf(stderr, "error finding %s: %s\n", serialNumber, usbErrorMessage(err));
    }
    return NULL;
  }
  
  return dev;
}

static void usage(char *myName)
{
  fprintf(stderr, "IWT313 host controller Version 0.0.3\n");
  fprintf(stderr, "2016 (C) Tokyo Devices, I.W. Technology Firm, Inc.\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "  %s list ... List all serial number of detected IWT313(s).\n", myName);
  fprintf(stderr, "  %s set <SerialNumber|\"ANY\"> <StateNumber> ... Set state of relays.\n", myName);
  fprintf(stderr, "  %s get <SerialNumber|\"ANY\">  ... Get state of relays.\n", myName);	
}

int main(int argc, char **argv)
{
  usbDevice_t *dev;
  char        buffer[17];    /* 16oCg̃obt@+|[gIDp1oCg */
  int         err;
  
  if(argc < 2) {
    usage(argv[0]);
    exit(1);
  }

  if(strcmp(argv[1], "list") == 0) {
    
    // IWT313̃VAԍ̃Xg1s1ŕ\܂		
    if ((dev = openDevice(NULL)) == NULL ) exit(1);
    
  } else if (strcmp(argv[1], "get") == 0) {
    // w肳ꂽVAԍ̃[̏Ԃ擾
    int buflen = sizeof(buffer);
    
    if (argc < 3) {
      fprintf(stderr, "Invalid arguments.\n");
      usage(argv[0]);
      exit(1);
    } else {
      // VAԍw肵ăfoCXJ
      if ((dev = openDevice(argv[2])) == NULL) exit(1);
      
      memset(buffer, 0, sizeof(buffer));
      if ((err = usbhidGetReport(dev, 0, buffer, &buflen)) != 0)
	fprintf(stderr, "error writing data: %s\n", usbErrorMessage(err));

      if (buffer[1] == 0x31 && buffer[2] == 0x7F) {
	printf("%u\n", (unsigned char)buffer[3]);
      } else {
	// ΉĂȂt@[EFAo[W
	fprintf(stderr, "not supported.\n");
      }
    }

  } else if(strcmp(argv[1], "set") == 0) {

    // w肳ꂽVAԍ̃[̏Ԃݒ肷
    if( argc < 4 ) {
      fprintf(stderr, "Invalid arguments.\n");
      usage(argv[0]);
      exit(1);
    } else  {
      // VAԍw肵USBfoCXJ
      if((dev = openDevice(argv[2])) == NULL) exit(1);
      
      // |[g쐬
      memset(buffer, 0, sizeof(buffer));
      buffer[1] = 0x81; // IWT313 - [Ԑݒ薽			
      buffer[2] = (char)atoi(argv[3]); // 8rbg̏ԕϐ
      
      if((err = usbhidSetReport(dev, buffer, sizeof(buffer))) != 0)   /* add a dummy report ID */
	fprintf(stderr, "error writing data: %s\n", usbErrorMessage(err));
    }
    
  } else {
    usage(argv[0]);
    exit(1);
  }
  
  // foCXN[Y
  usbhidCloseDevice(dev);
	
  return 0;
}
